
# Package
using LinearAlgebra
using Parameters
using IterativeSolvers
using FastGaussQuadrature
using ForwardDiff
using QuantEcon
using Plots
using Arpack
using BenchmarkTools
using JLD2
using MATLAB
using Plots; pyplot();
using SparseArrays;
using MAT
using Statistics

##
# Declation of variables and basic functions
# the structure is based on the EGM code of Alisdair McKay (errors ar ours)
struct AiyagariParametersEGM{T <: Real}
    β::T
    α::T
    δ::T
    γ::T
    Lbar::T
    χ::T
    ϕ::T
end

struct AiyagariModelEGM{T <: Real,I <: Integer}
    params::AiyagariParametersEGM{T}
    aGrid::Array{T,1} ##Policy grid
    aGridl::Array{T,1}
    na::I ##number of grid points in policy function
    dGrid::Array{T,1}
    nd::I ##number of grid points in distribution
    states::Array{T,1} ##earning states
    ns::I ##number of states
    Transv::Array{T,1}
end

mutable struct AggVarsEGM{S <: Real,T <: Real}
    R::S
    w::T
    TT::T # transfer
end


# be careful the following structure store information by bin (and not per capita). Divide by the size of the bin to have per capita terms

struct Projection{T <: Real,I <: Integer}
    Agg::AggVarsEGM{T,T}
    N::I
    ns::I ##number of states
    Nbin::I ##number of bins
    R::T
    w::T
    A::T
    B::T
    states::Array{T,1}
    Transv::Array{T,1}
    Sp::Array{T,1}
    abp::Array{T,1}
    aep::Array{T,1}
    lb::Array{T,1}
    cp::Array{T,1}
    Ucv::Array{T,1} # vector of marginal utilities (before multiplication by xsip) by bin
    Matab::SparseMatrixCSC{T,I}#Array{T,2}  # Transition Matrix for a  ( check abp =  Matab*aep)     spzeros(Nbin,Nbin)
    Ucp::Array{T,1}
    Ul::Array{T,1}
    Res::Array{T,1} #(check that resb is small and Res =  Upvb - β*R*Matab*Upvb
    xsip::Array{T,1}
    Ntoi::Array{I,1}
    CC::Array{T,1}
    indnc::Array{I,1} #Index of credit cosntrained histories
    indcc::Array{I,1} #Index of credit cosntrained histories
    ytype::Array{I,1}
end


# variables of the planner 
struct Plan{T <: Real}
    lambda::Array{T,1}
    lambdat::Array{T,1}
    psih::Array{T,1}
    omega::Array{T,1} # weights
    Uv::Array{T,1}
    CC::Array{T,1} # vector of residuals to must be 0 to  check equations
    mu::T
end

function gini2(wagedistarray)
    Swages = cumsum(wagedistarray[:,1].*wagedistarray[:,2])
    Gwages = Swages[1]*wagedistarray[1,2] +
                sum(wagedistarray[2:end,2] .*
                        (Swages[2:end]+Swages[1:end-1]))
    return 1 - Gwages/Swages[end]
end

uPrime(c,γ) = c.^(-γ)
uPrimeInv(up,γ) = up.^(-1.0/γ)

#

γ = 1.
if γ==1.0
    Uvv(X) = log.(X)+10
else
    Uvv(X) =  (X.^(1-γ)-1)./(1-γ)
end

function wealthD(ww::AbstractArray,NN::Integer)
ww2 = sortslices(ww, dims=1)
fracw = cumsum(ww2[:,1].*ww2[:,2])
fracw =fracw ./fracw[end]
fracS = cumsum(ww2[:,2])
plot(fracS,fracw)
vect = zeros(Float64,NN,2)
is  = 0
for i=1:NN
    is = maximum(findall(x->x<=i/NN,fracS))
    vect[i,1] = fracS[is] #- vect[i-1,1]
    vect[i,2] = fracw[is] #- vect[i-1,2]
end
vecf = zeros(Float64,NN,2)
vecf[1,:] .= vect[1,:]
for i=2:NN
    vecf[i,1] = vect[i,1] - vect[i-1,1]
    vecf[i,2] = vect[i,2] - vect[i-1,2]
end
return  vecf
end

##
# I here solve the steady-state before the truncation

##############################
#   MAIN
##############################
include("function_aiyagari2.jl")
include("parameters.jl")

tk,tl,tc = 0.36,0.28,0.00

KsY = 2.35*4
β = (1+KsY^(-1)*α-δ)^-1
Rt = 1/β #  before-tax gross interest rate
wt = (1-α)*( (Rt - (1-δ))/α )^(α/(α-1))
R = 1 + (1 - tk)*(Rt-1)
w = wt*(1-tl)/(1+tc) #correcting for consumption taxes (if any)
χ = ((1/3)^(1/ϕ))/w
Ltot = (χ*w)^ϕ

K = Ltot*( (Rt - (1-δ))/α )^(1/(α-1))
Y = K^α*Ltot^(1-α)

TTt = 0.08*Y   # if cost is TT=8% of GDP, agents receive TT

TT = TTt/(1+tc)
AA0,pol0,Aggs0 = AiyagariEGM(K,ns,Transv,states,amin,amax,curv,ϕ,χ,β,α,δ,γ,TT,lbar,na)
polA_ss,polC_ss,D_ss,A,Aggs_ss,Mat,upc = steady(pol0,AA0,R,w,TT,1e-4)
At = (1+tc)*A # correction for consumption taxes
Bt = At -K
B  = A - K
Ctot = w*Ltot + (R-1)*A + TT # = sum(polC_ss.*Dp_ss)
G = Y- δ*K - Ctot
@unpack dGrid = AA0
dGridl_t = repeat(dGrid,ns)*(1+tc)
D2 = copy(D_ss)
D2[1] = D_ss[1]
sum(D2)
ww = [dGridl_t D2]
ww2 = sortslices(ww, dims=1)
Gini = gini2(ww2)
vW = wealthD(ww2,5)

na = AA0.na
ns = AA0.ns
cpol = reshape(polC_ss,na,ns)
grida = reshape(dGridl_t,na,ns)
dist = reshape(D_ss,na,ns)
dc = cpol[2:na,:] .- cpol[1:na-1,:]
da = grida[2:na,:].- grida[1:na-1,:]
mpc = dc./da
ampc = sum(mpc.*(dist[1:na-1,:]))

println("B/Y: ",Bt/(Y*4), " Gini: ", Gini, " G/Y ",G/Y)
vW = wealthD(ww2,10)

##
include("projection_trunc.jl")
include("function_optimis_hat.jl")
N  = 5 # truncation length
nc = 81 # number of credit constrained history

@time eco,upc,resE, stdΓ,resi,ytype = Project_plan(N,AA0,polA_ss,polC_ss,upc,D_ss,Aggs_ss,Mat,A,B,vstan,states,nc)
plot(eco.xsip)
stdΓ

Pl = Optim(eco,AA0.params)

# computing the pricing kernel
M = sum(eco.Ucv.*eco.xsip.*Pl.omega.*eco.Sp)
#saving todynare
file = matopen("todynaremone.mat", "w")
write(file, "Pl", Pl)
write(file, "eco", eco)
write(file, "alpha", AA0.params.α)
write(file, "beta", AA0.params.β)
write(file, "delta", AA0.params.δ)
write(file, "gamma", AA0.params.γ)
write(file, "chi", AA0.params.χ)
write(file, "phi", AA0.params.ϕ)
write(file, "abar", minimum(AA0.aGrid))
write(file, "states", AA0.states)
write(file, "G", G)
write(file, "tk", tk)
write(file, "tl", tl)
write(file, "Ctot", Ctot)
write(file, "M", M)
close(file)
