%%%%%%%% Generate a latex table wit relevant information for dynare running
clear

load totableRA oo_
res(1).mean = oo_.mean;
res(1).var = oo_.var;
res(1).autocorr = oo_.autocorr;
load  totalbeaggreal oo_
res(2).mean = oo_.mean;
res(2).var = oo_.var;
res(2).autocorr = oo_.autocorr;
load totableaggnk oo_
res(3).mean = oo_.mean;
res(3).var = oo_.var;
res(3).autocorr = oo_.autocorr;


% Order (C=13 in oo_):  Ctot K GDP Ltot tauk taul B T
%% stadard deviation of TFP shock in the simulations (the three simulations)
stdTFP = res(1).var(12,12)^0.5;
stdu = 0.0031/( (1-0.95)^0.5);
corr = stdu/stdTFP;

table = zeros(25,3);
for i=1:8
    for j=1:3
        mean = res(j).mean(i+12);
        sstd = (res(j).var(i+12,i+12))^.5;
        table((i-1)*2+1,j)=mean;
        if or(i==6,i==5)
            table((i-1)*2+2,j)=sstd*corr;        
        else
            table((i-1)*2+2,j)=sstd*corr/abs(mean);        
        end
    end
end
i=16;

%%inflation
table(i+1,1)=0;
table(i+2,1)=0;
table(i+1,2)=0;
table(i+2,2)=0;
table(i+1,3)=0;
table(i+2,3)=corr*(res(3).var(12,12))^.5;
i=18;

%%% correlation
for k=17:20
 for j=1:3
        corrt = res(j).var(k,15)/ ( res(j).var(k,k)*res(j).var(15,15))^.5; % 15 is GDP
        table(i+k-16,j)=corrt;           
    end
end
table(19,3)=0;

i=22;
 for j=1:3
        corrt = res(j).var(13,15)/ ( res(j).var(13,13)*res(j).var(15,15))^.5; % 15 is GDP
        table(23,j)=corrt;           
    end;   

%%% Autocorrelation
for j=1:3     
    table(24,j)=res(j).autocorr{1,1}(15,15);       %GDP 
    table(25,j)=res(j).autocorr{1,1}(19,19);      %debt
end; 
 

clear input;

% numeric values you want to tabulate:
% this field has to be an array or a MATLAB table
% in this example we use an array
% input.data = [1.12345 NaN 3.12345; ...
%     4.12345 5.12345 6.12345; ...
%     7.12345 8.12345 9.12345; ...
%     10.12345 11.12345 12.12345];

input.data = table;
% Optional fields:

% Set column labels (use empty string for no label):
input.tableColLabels = {'RA','Full','No cap. tax'};
% Set row labels (use empty string for no label):
input.tableRowLabels = {'C mean','  std','K mean','  std','Y mean','  std','L mean',...
    '  std','$\tau^{K}$ mean','  std','$\tau^{L}$ mean','  std','B mean','  std','T mean','  std','$\pi$ mean','  std',...
     'corr($\tau^{K}$,Y)','corr($\tau^{L}$,Y)','corr(B,Y)','corr(T,Y)','corr(C,Y)','corr(Y,Y$_{-1}$)','corr(B,B$_{-1}$)'};

% Switch transposing/pivoting your table:
input.transposeTable = 0;

% Determine whether input.dataFormat is applied column or row based:
input.dataFormatMode = 'column'; % use 'column' or 'row'. if not set 'colum' is used

% Formatting-string to set the precision of the table values:
% For using different formats in different rows use a cell array like
% {myFormatString1,numberOfValues1,myFormatString2,numberOfValues2, ... }
% where myFormatString_ are formatting-strings and numberOfValues_ are the
% number of table columns or rows that the preceding formatting-string applies.
% Please make sure the sum of numberOfValues_ matches the number of columns or
% rows in input.tableData!
%
input.dataFormat = {'%.4f',2,'%.4f',1}; % three digits precision for first two columns, one digit for the last

% Define how NaN values in input.tableData should be printed in the LaTex table:
input.dataNanString = '-';

% Column alignment in Latex table ('l'=left-justified, 'c'=centered,'r'=right-justified):
input.tableColumnAlignment = 'l';

% Switch table borders on/off (borders are enabled by default):
input.tableBorders = 0;

% Uses booktabs basic formating rules ('1' = using booktabs, '0' = not using booktabs). 
% Note that this option requires the booktabs package being available in your LaTex. 
% Also, setting the booktabs option to '1' overwrites input.tableBorders if it exists.
% input.booktabs = 0;


% LaTex table caption:
input.tableCaption = 'MyTableCaption';

% LaTex table label:
input.tableLabel = 'MyTableLabel';

% Switch to generate a complete LaTex document or just a table:
input.makeCompleteLatexDocument = 1;

% call latexTable:
latex = latexTable(input);
